@@ -49,10 +49,7 @@
 #include "nsCRT.h"
 
 #if defined(XP_MACOSX)
-#include <Folders.h>
-#include <Script.h>
-#include <Processes.h>
-#include <Gestalt.h>
+#include <Carbon/Carbon.h>
 #include "nsILocalFileMac.h"
 #elif defined(XP_OS2)
 #define INCL_DOSPROCESS
@@ -65,10 +62,6 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <sys/param.h>
-#elif defined(XP_BEOS)
-#include <sys/param.h>
-#include <kernel/image.h>
-#include <storage/FindDirectory.h>
 #endif
 
 
@@ -94,7 +87,7 @@
 #ifdef XP_MACOSX
 #define NS_MACOSX_USER_PLUGIN_DIR   "OSXUserPlugins"
 #define NS_MACOSX_LOCAL_PLUGIN_DIR  "OSXLocalPlugins"
-#define NS_MAC_CLASSIC_PLUGIN_DIR   "MacSysPlugins"
+#define NS_MACOSX_JAVA2_PLUGIN_DIR  "OSXJavaPlugins"
 #elif XP_UNIX
 #define NS_SYSTEM_PLUGINS_DIR       "SysPlugins"
 #endif
@@ -136,9 +129,7 @@ nsAppFileLocationProvider::GetFile(const char *prop, PRBool *persistent, nsIFile
     *persistent = PR_TRUE;
 
 #ifdef XP_MACOSX
-    short foundVRefNum;
-    long foundDirID;
-    FSSpec fileSpec;
+    FSRef fileRef;
     nsCOMPtr<nsILocalFileMac> macFile;
 #endif
     
@@ -206,36 +197,25 @@ nsAppFileLocationProvider::GetFile(const char *prop, PRBool *persistent, nsIFile
 #ifdef XP_MACOSX
     else if (nsCRT::strcmp(prop, NS_MACOSX_USER_PLUGIN_DIR) == 0)
     {
-        if (!(::FindFolder(kUserDomain,
-                           kInternetPlugInFolderType,
-                           kDontCreateFolder, &foundVRefNum, &foundDirID)) &&
-            !(::FSMakeFSSpec(foundVRefNum, foundDirID, "\p", &fileSpec))) {
-            rv = NS_NewLocalFileWithFSSpec(&fileSpec, PR_TRUE, getter_AddRefs(macFile));
+        if (::FSFindFolder(kUserDomain, kInternetPlugInFolderType, false, &fileRef) == noErr) {
+            rv = NS_NewLocalFileWithFSRef(&fileRef, PR_TRUE, getter_AddRefs(macFile));
             if (NS_SUCCEEDED(rv))
                 localFile = macFile;
         }
     }
     else if (nsCRT::strcmp(prop, NS_MACOSX_LOCAL_PLUGIN_DIR) == 0)
     {
-        if (!(::FindFolder(kLocalDomain,
-                           kInternetPlugInFolderType,
-                           kDontCreateFolder, &foundVRefNum, &foundDirID)) &&
-            !(::FSMakeFSSpec(foundVRefNum, foundDirID, "\p", &fileSpec))) {
-            rv = NS_NewLocalFileWithFSSpec(&fileSpec, PR_TRUE, getter_AddRefs(macFile));
+        if (::FSFindFolder(kLocalDomain, kInternetPlugInFolderType, false, &fileRef) == noErr) {
+            rv = NS_NewLocalFileWithFSRef(&fileRef, PR_TRUE, getter_AddRefs(macFile));
             if (NS_SUCCEEDED(rv))
                 localFile = macFile;
         }
     }
-    else if (nsCRT::strcmp(prop, NS_MAC_CLASSIC_PLUGIN_DIR) == 0)
+    else if (nsCRT::strcmp(prop, NS_MACOSX_JAVA2_PLUGIN_DIR) == 0)
     {
-        if (!(::FindFolder(kOnAppropriateDisk,
-                           kInternetPlugInFolderType,
-                           kDontCreateFolder, &foundVRefNum, &foundDirID)) &&
-            !(::FSMakeFSSpec(foundVRefNum, foundDirID, "\p", &fileSpec))) {
-            rv = NS_NewLocalFileWithFSSpec(&fileSpec, PR_TRUE, getter_AddRefs(macFile));
-            if (NS_SUCCEEDED(rv))
-                localFile = macFile;
-        }
+      static const char *const java2PluginDirPath =
+        "/System/Library/Frameworks/JavaVM.framework/Versions/Current/Resources/";
+      NS_NewNativeLocalFile(nsDependentCString(java2PluginDirPath), PR_TRUE, getter_AddRefs(localFile));
     }
 #else
     else if (nsCRT::strcmp(prop, NS_ENV_PLUGINS_DIR) == 0)
@@ -378,17 +358,6 @@ NS_METHOD nsAppFileLocationProvider::GetProductDirectory(nsILocalFile **aLocalFi
 #elif defined(XP_UNIX)
     rv = NS_NewNativeLocalFile(nsDependentCString(PR_GetEnv("HOME")), PR_TRUE, getter_AddRefs(localDir));
     if (NS_FAILED(rv)) return rv;
-#elif defined(XP_BEOS)
-    char path[MAXPATHLEN];
-    find_directory(B_USER_SETTINGS_DIRECTORY, 0, 0, path, MAXPATHLEN);
-    // Need enough space to add the trailing backslash
-    int len = strlen(path);
-    if (len > MAXPATHLEN-2)
-        return NS_ERROR_FAILURE;
-    path[len]   = '/';
-    path[len+1] = '\0';
-    rv = NS_NewNativeLocalFile(nsDependentCString(path), PR_TRUE, getter_AddRefs(localDir));
-    if (NS_FAILED(rv)) return rv;
 #else
 #error dont_know_how_to_get_product_dir_on_your_platform
 #endif
@@ -517,7 +486,7 @@ NS_IMPL_ISUPPORTS1(nsAppDirectoryEnumerator, nsISimpleEnumerator)
 
 #if defined(XP_WIN) || defined(XP_OS2) /* Win32 and OS/2 */
 #define PATH_SEPARATOR ';'
-#else /*if defined(XP_UNIX) || defined(XP_BEOS)*/
+#else
 #define PATH_SEPARATOR ':'
 #endif
 
@@ -584,17 +553,11 @@ nsAppFileLocationProvider::GetFiles(const char *prop, nsISimpleEnumerator **_ret
     if (!nsCRT::strcmp(prop, NS_APP_PLUGINS_DIR_LIST))
     {
 #ifdef XP_MACOSX
-        static const char* osXKeys[] = { NS_APP_PLUGINS_DIR, NS_MACOSX_USER_PLUGIN_DIR, NS_MACOSX_LOCAL_PLUGIN_DIR, nsnull };
-        static const char* os9Keys[] = { NS_APP_PLUGINS_DIR, NS_MAC_CLASSIC_PLUGIN_DIR, nsnull };
-        static const char** keys;
-        
-        if (!keys) {
-            OSErr err;
-            long response;
-            err = ::Gestalt(gestaltSystemVersion, &response); 
-            keys = (!err && response >= 0x00001000) ? osXKeys : os9Keys;
-        }
-
+        // We are temporarily looking for JavaPlugin2 in the Java framework because Apple did not want
+        // to enable it for Safari by including it in the normal search directories. This situation
+        // should be resolved soon and then we should stop looking for JavaPlugin2 explicitly.
+        static const char* keys[] = { NS_APP_PLUGINS_DIR, NS_MACOSX_USER_PLUGIN_DIR,
+                                      NS_MACOSX_LOCAL_PLUGIN_DIR, NS_MACOSX_JAVA2_PLUGIN_DIR, nsnull };
         *_retval = new nsAppDirectoryEnumerator(this, keys);
 #else
 #ifdef XP_UNIX
